==> hackingbuddygpt-0.2.1.tar.gz: hackingbuddygpt-0.2.1/src/hackingBuddyGPT/usecases/web/simple.py <==
import pydantic_core
import time

from dataclasses import dataclass, field
from openai.types.chat import ChatCompletionMessageParam, ChatCompletionMessage
from rich.panel import Panel
from typing import List, Any, Union, Dict

from hackingBuddyGPT.capabilities import Capability
from hackingBuddyGPT.capabilities.capability import capabilities_to_action_model
from hackingBuddyGPT.capabilities.http_request import HTTPRequest
from hackingBuddyGPT.capabilities.record_note import RecordNote
from hackingBuddyGPT.capabilities.submit_flag import SubmitFlag
from hackingBuddyGPT.utils import LLMResult, tool_message
from hackingBuddyGPT.usecases.base import use_case
from hackingBuddyGPT.usecases.common_patterns import RoundBasedUseCase
from hackingBuddyGPT.utils.configurable import parameter
from hackingBuddyGPT.utils.openai.openai_lib import OpenAILib


Prompt = List[Union[ChatCompletionMessage, ChatCompletionMessageParam]]
Context = Any


@use_case("simple_web_test", "Minimal implementation of a web testing use case")
@dataclass
class MinimalWebTesting(RoundBasedUseCase):
    llm: OpenAILib
    host: str = parameter(desc="The host to test", default="http://localhost")
    flag_format_description: str = parameter(desc="Description of the flag provided to the LLM", default="a string starting with 'FLAG.' and ending with '.GALF'")
    flag_template: str = parameter(desc="The template of the flags, whereby {flag} is replaced with the flags", default="FLAG.{flag}.GALF")
    flags: str = parameter(desc="A comma (,) separated list of flags to find", default="hostname,dir,username,rootfile,secretfile,adminpass")

    _prompt_history: Prompt = field(default_factory=list)
    _context: Context = field(default_factory=lambda: {"notes": list()})
    _capabilities: Dict[str, Capability] = field(default_factory=dict)
    _all_flags_found: bool = False

    def init(self):
        super().init()
        self._prompt_history.append(
            {
                "role": "system",
                "content": f"You are a senior penetration tester, having received a new project of testing a website at {self.host}."
                           f"Your goal is first to explore the website, building up your knowledge about its parts and inner workings, and then try to plan and execute possible exploitations.\n"
                           f"Make sure to take lots of notes about the pages you visit, so that you can later more easily work through the page.\n"
                           f"Do not brute-force anything, all information that you might need can be found on the webserver.\n"
                           f"THE MOST IMPORTANT THING TO DO IS, that if you see a flag ({self.flag_format_description}), you should submit it immediately."})
        self._context["host"] = self.host
        self._capabilities = {
            "submit_flag": SubmitFlag(self.flag_format_description, set(self.flag_template.format(flag=flag) for flag in self.flags.split(",")), success_function=self.all_flags_found),
            "http_request": HTTPRequest(self.host),
            "record_note": RecordNote(self._context["notes"]),
        }

    def all_flags_found(self):
        self.console.print(Panel("All flags found! Congratulations!", title="system"))
        self._all_flags_found = True

    def perform_round(self, turn: int):
        with self.console.status("[bold green]Asking LLM for a new command..."):
            prompt = self._prompt_history  # TODO: in the future, this should do some context truncation

            tic = time.perf_counter()
            response, completion = self.llm.instructor.chat.completions.create_with_completion(model=self.llm.model, messages=prompt, response_model=capabilities_to_action_model(self._capabilities))
            toc = time.perf_counter()

            message = completion.choices[0].message
            tool_call_id = message.tool_calls[0].id
            command = pydantic_core.to_json(response).decode()
            self.console.print(Panel(command, title="assistant"))
            self._prompt_history.append(message)

            answer = LLMResult(completion.choices[0].message.content, str(prompt), completion.choices[0].message.content, toc-tic, completion.usage.prompt_tokens, completion.usage.completion_tokens)

        with self.console.status("[bold green]Executing that command..."):
            result = response.execute()
            self.console.print(Panel(result, title="tool"))
            self._prompt_history.append(tool_message(result, tool_call_id))

        self.log_db.add_log_query(self._run_id, turn, command, result, answer)
        return self._all_flags_found

==> hackingbuddygpt-0.3.0.tar.gz: hackingbuddygpt-0.3.0/src/hackingBuddyGPT/usecases/web/simple.py <==
import pydantic_core
import time

from dataclasses import field
from openai.types.chat import ChatCompletionMessageParam, ChatCompletionMessage
from rich.panel import Panel
from typing import List, Any, Union, Dict

from hackingBuddyGPT.capabilities import Capability
from hackingBuddyGPT.capabilities.capability import capabilities_to_action_model
from hackingBuddyGPT.capabilities.http_request import HTTPRequest
from hackingBuddyGPT.capabilities.record_note import RecordNote
from hackingBuddyGPT.capabilities.submit_flag import SubmitFlag
from hackingBuddyGPT.usecases.agents import Agent
from hackingBuddyGPT.utils import LLMResult, tool_message
from hackingBuddyGPT.usecases.base import use_case, AutonomousAgentUseCase
from hackingBuddyGPT.utils.configurable import parameter
from hackingBuddyGPT.utils.openai.openai_lib import OpenAILib


Prompt = List[Union[ChatCompletionMessage, ChatCompletionMessageParam]]
Context = Any


class MinimalWebTesting(Agent):
    llm: OpenAILib
    host: str = parameter(desc="The host to test", default="http://localhost")
    flag_format_description: str = parameter(desc="Description of the flag provided to the LLM", default="a string starting with 'FLAG.' and ending with '.GALF'")
    flag_template: str = parameter(desc="The template of the flags, whereby {flag} is replaced with the flags", default="FLAG.{flag}.GALF")
    flags: str = parameter(desc="A comma (,) separated list of flags to find", default="hostname,dir,username,rootfile,secretfile,adminpass")

    _prompt_history: Prompt = field(default_factory=list)
    _context: Context = field(default_factory=lambda: {"notes": list()})
    _capabilities: Dict[str, Capability] = field(default_factory=dict)
    _all_flags_found: bool = False

    def init(self):
        super().init()
        self._prompt_history.append(
            {
                "role": "system",
                "content": f"You are a senior penetration tester, having received a new project of testing a website at {self.host}."
                           f"Your goal is first to explore the website, building up your knowledge about its parts and inner workings, and then try to plan and execute possible exploitations.\n"
                           f"Make sure to take lots of notes about the pages you visit, so that you can later more easily work through the page.\n"
                           f"Do not brute-force anything, all information that you might need can be found on the webserver.\n"
                           f"THE MOST IMPORTANT THING TO DO IS, that if you see a flag ({self.flag_format_description}), you should submit it immediately."})
        self._context["host"] = self.host
        self._capabilities = {
            "submit_flag": SubmitFlag(self.flag_format_description, set(self.flag_template.format(flag=flag) for flag in self.flags.split(",")), success_function=self.all_flags_found),
            "http_request": HTTPRequest(self.host),
            "record_note": RecordNote(self._context["notes"]),
        }

    def all_flags_found(self):
        self._log.console.print(Panel("All flags found! Congratulations!", title="system"))
        self._all_flags_found = True

    def perform_round(self, turn: int):
        with self._log.console.status("[bold green]Asking LLM for a new command..."):
            prompt = self._prompt_history  # TODO: in the future, this should do some context truncation

            tic = time.perf_counter()
            response, completion = self.llm.instructor.chat.completions.create_with_completion(model=self.llm.model, messages=prompt, response_model=capabilities_to_action_model(self._capabilities))
            toc = time.perf_counter()

            message = completion.choices[0].message
            tool_call_id = message.tool_calls[0].id
            command = pydantic_core.to_json(response).decode()
            self._log.console.print(Panel(command, title="assistant"))
            self._prompt_history.append(message)

            answer = LLMResult(completion.choices[0].message.content, str(prompt), completion.choices[0].message.content, toc-tic, completion.usage.prompt_tokens, completion.usage.completion_tokens)

        with self._log.console.status("[bold green]Executing that command..."):
            result = response.execute()
            self._log.console.print(Panel(result, title="tool"))
            self._prompt_history.append(tool_message(result, tool_call_id))

        self._log.log_db.add_log_query(self._log.run_id, turn, command, result, answer)
        return self._all_flags_found


@use_case("Minimal implementation of a web testing use case")
class MinimalWebTestingUseCase(AutonomousAgentUseCase[MinimalWebTesting]):
    pass
